<template>
    
  <div class="input__container w-100" :class="containerClasses" :style="inputContainerStyle">
    <div class="slot-container" ref="input-container">
      <slot
        v-model="value"
        @focus="focus"
        @blur="blur"
      class="input"></slot>
    </div>
    <label class="label__placeholder" :for="labelName">{{ config.label }}</label>
    <label class="label__active" :class="activeLabelClasses" :style="activeLabelStyle" :for="labelName">{{ config.label }}</label>
  </div>
    
</template>

<script>
    module.exports = {
        name: 'FloatingLabel',
        data () {
            return {
                defaultSettings: {
                    classes: {
                        error: 'has-error'
                    },
                    height: 50,
                    scale: true,
                    labelOffset: {
                        top: 7,
                        left: 17.5
                    },
                    color: {
                        focusColor: '#299065',
                        errorColor: '#f05d61',
                        blurredColor: '#777'
                    }
                },
                hasFocus: false,
                hasContent: false
            }
        },
        props: {
            config: {
                required: true
            },
            hasError: {
                type: Boolean,
                default: false
            },
            model: {
                
            }
        },
        computed: {
            activeLabelClasses() {
                return {
                    'label__active--canscale': this.settings.scale
                }
            },
            containerClasses() {
                let classes = {
                    'input__container--focus': this.hasFocus,
                    'input__container--content': this.hasContent
                };
                if (this.settings.hasError) {
                    classes[this.settings.classes.error] = true;
                }
                return classes;
            },
            labelName() {
                if (this.config.name !== undefined) {
                    return this.config.name
                }
                return this.config.label.toLowerCase();
            },
            labelColor() {
                if (!this.hasError) {
                    return this.hasFocus ? this.settings.color.focusColor : this.settings.color.blurredColor
                } else {
                    return this.settings.color.errorColor;
                }
            },
            activeLabelStyle() {
                return {
                    top: this.settings.labelOffset.top + 'px',
                    left: this.settings.labelOffset.left + 'px',
                    color: this.labelColor
                }
            },
            inputContainerStyle() {
                return {
                    height: this.settings.height + 'px'
                }
            },
            settings() {
                return Object.assign({}, this.defaultSettings, this.config);
            }
        },
        methods: {
            clear() {
                this.formElement.value = '';
                this.hasContent = false;
                this.hasFocus = false;
                this.$emit('clear');
            },
            focus(event) {
                this.hasFocus = true;
                this.$emit('focus');
            },
            input(event) {
                this.hasFocus = true;
                this.hasContent = event.target.value !== '';
                this.$emit('input');
            },
            blur(event) {
                this.hasFocus = false;
                this.$emit('blur');
            },
            checkPrefill(){
              if(this.model && this.model.length > 0){
                this.hasFocus = false;
                this.hasContent = true;
              }
            }
        },
        mounted() {
            this.checkPrefill();
            this.formElement = this.$refs['input-container'].querySelector('input, select');
            if (this.formElement) {
                this.formElement.addEventListener('input', this.input);
                this.formElement.addEventListener('blur', this.blur);
                this.formElement.addEventListener('focus', this.focus);
                if (this.formElement.type === 'select-one') {
                    this.hasContent = true;
                    this.settings.scale = false;
                }
            }
        },
        watch: {
          model: function(){
            this.checkPrefill();
          },
        }
    }
</script>

<style scoped>

  input{ height: 50px !important; padding: 5px 0 0 15px !important; }

  select{ padding: 15px 10px 5px !important; }

  label {
    position: absolute;
    top: 0;
    left: 0;
    pointer-events: none;
    -webkit-user-select: none;
    -moz-user-select: none;
    -ms-user-select: none;
    user-select: none;
    -webkit-transition: 0.2s cubic-bezier(0.645, 0.045, 0.355, 1);
    -moz-transition: 0.2s cubic-bezier(0.645, 0.045, 0.355, 1);
    transition: 0.2s cubic-bezier(0.645, 0.045, 0.355, 1);
  }
  label.label__placeholder {
    top: 14px;
    color: rgba(3, 23, 40, 0.4);
    left: 18px;
    white-space: nowrap;
    width: calc(100% - 22.5px);
    text-overflow: ellipsis;
    overflow: hidden;
  }
  label.label__active {
    font-size: 13px;
    color: rgba(3, 23, 40, 0.34);
    font-weight: 500;
    line-height: 16px;
    top: 14px;
    opacity: 0;
    left: 15px;
    overflow: hidden;
    white-space: nowrap;
    text-overflow: ellipsis;
    width: calc(100% - 22.5px);
  }
/*  label.label__passive {
    font-size: 11px;
    color: rgba(3, 23, 40, 0.44);
    line-height: 8px;
    top: 16px;
    opacity: 0;
  }*/

  textarea {
    resize: none;
    padding: 0;
    border: 0;
    width: 100%;
    padding-top: 20px;
    padding-right: 40px;
    padding-bottom: 0;
    display: block;
    box-sizing: padding-box;
    overflow: hidden;
    background-color: transparent;
    margin: 0;
    -webkit-transition: 0.2s cubic-bezier(0.645, 0.045, 0.355, 1);
    -moz-transition: 0.2s cubic-bezier(0.645, 0.045, 0.355, 1);
    transition: 0.2s cubic-bezier(0.645, 0.045, 0.355, 1);
  }

  .input__container {
    position: relative;
    -webkit-transition: 0.2s cubic-bezier(0.645, 0.045, 0.355, 1);
    -moz-transition: 0.2s cubic-bezier(0.645, 0.045, 0.355, 1);
    transition: 0.2s cubic-bezier(0.645, 0.045, 0.355, 1);
    overflow: hidden;
  }
  .input__container.error__active {
    margin-bottom: 32px;
  }
  .input__container.error__active.input__container--content {
    padding-bottom: 0;
  }
  .input__container + .input__container:before {
    display: none;
  }
  .input__container.input__container--focus .label__active {
    color: #128CED;
  }
  .input__container.input__container--content label.label__placeholder {
    opacity: 0;

  }
  .input__container.input__container--content .label__active {
    opacity: 1;
    font-size: 11px;
  }
  .input__container.input__container--content.error__active {
    margin-bottom: 0;
  }
  .input__container.input__container--content .error__message {
    opacity: 0;
    -webkit-transform: translate3d(0, -4px, 0);
    -moz-transform: translate3d(0, -4px, 0);
    transform: translate3d(0, -4px, 0);
  }
  .input__container.input__container--focus.input__container--content label.label__active {
    opacity: 1;
    -webkit-transform: translate3d(0, 0px, 0);
    -moz-transform: translate3d(0, 0px, 0);
    transform: translate3d(0, 0px, 0);
  }
  .input__container.input__container--focus.input__container--content label.label__placeholder, .input__container.input__container--focus.input__container--content label.label__passive {
    opacity: 0;
  }
  .input__container.input__container--focus.input__container--content textarea {
    -webkit-transform: translate3d(0, 12px, 0);
    -moz-transform: translate3d(0, 12px, 0);
    transform: translate3d(0, 12px, 0);
  }

</style>